%% Parameters
V_hex = 100;            %Heat exchanger volume (shell or tube side) [l]
V_tank = 1000e3;         %Storage tank volume [l]
U_hex = 0.50; %0.32     %Overall heat transfer coefficient [kW/m^2K]
A_hex = 300; %200       %Heat transfer area [m^2]
rho = 1;                 %Density [kg/l]
cp = 4.186;             %specific heat capacity of fluid [kJ/kgK]
h_s = 0.050e-3;         %heat loss coefficient [kW/m^2K]
n = 1/3; %1/3;          %approximation index
A_tank =1000;           %Tank heat loss area [m^2]
Pm =1000e-6;
Pt =5e-6;
Pu = 50e-6;
Px = 5e-6;
Q_demand_0 = 5000;
tol = 1e-6;
maxiter=2e4;
std = 2.31;             %noise standard deviation

%% Disturbances (Boundary conditions)
T1 = 95;                %Plant 1 temperature (source)[degC]
T2 = 20;                %Plant 2 temperature (sink)[degC]
T_s = 15;               %Ambient temperature [degC]

T_L1req = 0;

q_L1 = 50;
q_R1 = 50;

%% Bounds
T_L1ub = 200; 
T_R1ub = 200;
T_L2ub = 200;
T_R2ub = 200;
T_tankub = 150;
q_L1ub = 50;
q_R1ub = 50;
q_L2ub = 50;
q_R2ub = 50;
Q_tankub = 0;
Qmub = Inf;
Qdub = Inf;

T_L1lb = T_L1req; 
T_R1lb = 0;
T_L2lb = 0;
T_R2lb = 0;
T_tanklb = 30;
q_L1lb = 0;
q_R1lb = 0;
q_L2lb = 0;
q_R2lb = 0;
Q_tanklb = 0;
Qmlb = 0;
Qdlb = 0;

xub = vertcat(T_L1ub, T_R1ub, T_L2ub, T_R2ub, T_tankub);
xlb = vertcat(T_L1lb, T_R1lb, T_L2lb, T_R2lb, T_tanklb);
uub = vertcat(q_L2ub, q_R2ub, Q_tankub, Qmub);
ulb = vertcat(q_L2lb, q_R2lb, Q_tanklb, Qmlb);

%% Initial values
x_init = [80;60;40;30;50]; %[85; 75; 55;45; 65];%[50; 40; 20; 10; 30];
x_0 = x_init;
u_init = [25;25;0;0];
u_n1 = u_init;
z_init = 3*ones(4,1);
h_dot = (U_hex*A_hex)/(rho*cp);
h_t_dot = (h_s*A_tank)/(rho*cp);


