% An implementation of direct collocation to open loop 
% dynamic optimisation of a two plant energy Storage System using CasADi
% VARIABLE DEMAND!!! with plant model mismatch an implementation of
% multi-stage MPC to handle uncertainties.

% Author: Zawadi Mdoe 
% Date: March 2019
% ============================================================================

clear;
clc;
close all;

addpath('C:\Users\DELL\Desktop\Matlab\casadi-windows-matlabR2016a-v3.4.5')
import casadi.*
run Parameters_scendae2.m
% load uncertaintyparametersdata.mat
load uncertaintyparametersdata_uniform.mat
%load Expecteddemand.mat
% load CaliforniaDemand.mat
load CaliforniaDemand2.mat
% load CaliforniaDemand3.mat

% Uncertain parameter(s)
par1 = [85; 90; 95];   %Source temperature
par2 = [15; 20; 25];    %Sink temperature
%par3 = [10; 20];

[scens, scen_count] = scenpara(par1,par2);

%Parameter distribution
% alpha = 0.05;                   %confidence level
% zscore = norminv(1-alpha);
% std = [(max(abs(par1)) - mean(par1));
%        23 (max(abs(par2)) - mean(par2))]/zscore;

% Degree of interpolating polynomial
d = 3;

% Get collocation points
tau_root = [0 collocation_points(d, 'legendre')]; 

% Coefficients of the collocation equation
C = zeros(d+1,d+1);

% Coefficients of the continuity equation
D = zeros(d+1, 1);

% Coefficients of the quadrature function
B = zeros(d+1, 1);

% Construct polynomial basis
for j=1:d+1
  % Construct Lagrange polynomials to get the polynomial basis at the collocation point
  coeff = 1;
  for r=1:d+1
    if r ~= j
      coeff = conv(coeff, [1, -tau_root(r)]);                 %convolution
      coeff = coeff / (tau_root(j)-tau_root(r));
    end
  end
  % Evaluate the polynomial at the final time to get the coefficients of the continuity equation
  D(j) = polyval(coeff, 1.0);

  % Evaluate the time derivative of the polynomial at all collocation points to get the coefficients of the continuity equation
  pder = polyder(coeff);
  for r=1:d+1
    C(j,r) = polyval(pder, tau_root(r));
  end

  % Evaluate the integral of the polynomial to get the coefficients of the quadrature function
  pint = polyint(coeff);
  B(j) = polyval(pint, 1.0);
end

% Time horizon
T = 24*60*60;               %Prediction horizon time
ndiff = 5;                  %number of differential states
nalg = 4;                   %number of algebraic states
nu = 6;                     %number of controls
nx = nalg + ndiff;          %total number of states

zub = Inf*ones(nalg,1);
zlb = -Inf*ones(nalg,1);

% Declare model variables
x1 = SX.sym('x1');
x2 = SX.sym('x2');
x3 = SX.sym('x3');
x4 = SX.sym('x4');
x5 = SX.sym('x5');
x = [x1; x2; x3; x4; x5];
x6 = SX.sym('x6');
x7 = SX.sym('x7');
x8 = SX.sym('x8');
x9 = SX.sym('x9');
z = [x6; x7; x8; x9];
u1 = SX.sym('u1');
u2 = SX.sym('u2');
u3 = SX.sym('u3');
u4 = SX.sym('u4');
u5 = SX.sym('u5');
u6 = SX.sym('u6');
u = [u1; u2; u3; u4; u5; u6];
p1 = SX.sym('p1');
p2 = SX.sym('p2');
p = [p1; p2];

% Model equations
xdot =  [(1/V_hex)*(u1*(p1-x1) - h_dot*(0.5*(x6 + x7))^(1/n));

        (1/V_hex)*(u2*(x5-x2) + h_dot*(0.5*(x6 + x7))^(1/n));

        (1/V_hex)*(u3*(x5-x3) - h_dot*(0.5*(x8 + x9))^(1/n));

        (1/V_hex)*(u4*(p2-x4) + h_dot*(0.5*(x8 + x9))^(1/n));
        
        (1/V_tank)*(u2*(x2-x5) + u3*(x3-x5) + (u5/(rho*cp)-h_t_dot*(x5-T_s)));
        
        p1-x2- x6^(1/n);
        
        x1-x5-x7^(1/n);
        
        x3-p2-x8^(1/n);
        
        x5-x4-x9^(1/n)];

% Objective term
L = Pm*u6 + Pt*u5 + Pu*(u1^2+ u2^2+ u3^2+ u4^2);

% Continuous time dynamics
f = Function('f', {x, z, u, p}, {xdot, L});

% Control discretization
N = 24;         % number of control intervals
M = 24;         %mpc loops
h = T/N;
Nr = 1;
levels = scen_count;
S = levels^Nr;  % number of scenarios
period = N;
Q_demand = [CalDemand; CalDemand];
%Q_demand = 6000*ones(N+M,1);
% Q_demand = zeros(N+M,1);


% Variable counting
NXD = N*ndiff*(d+1);    %Total Number of differential state variables 
NXA = N*nalg*d;         %Total Number of algebraic state variables
NU = N*nu;              %Total Number of input variables
NXF = ndiff;            %Total Number of end point variables (differential only)
NV = NXD + NXA + NU + NXF; %Total number of NLP variables

% Prepare output variables
x_opt = zeros(M,ndiff);
u_opt = NaN(M+1,nu);
i_infeasible = zeros(M,1);
solver_return = cell(1,M);
w_stored = zeros(NV,1);

%% MPC loop
for ii=1:M
    if ii == 1 
        % Start with an empty NLP
        w={};
        w0 = [];
        lbw = [];
        ubw = [];
        J = 0;
        g={};
        lbg = [];
        ubg = [];

        % "Lift" initial conditions
        Xkl = MX.sym('X0', ndiff);
        w = [w(:)', {Xkl}];
        lbw = [lbw; x_init];
        ubw = [ubw; x_init];    
        w0 = [w0; x_init];

        % Formulate the NLP
        % For each scenario 
        for l=1:S
            % New NLP variable for the control
            for k=0:N-1
                Ukl = MX.sym(['U_' num2str(k) '_' num2str(l)], nu);
                w = [w(:)', {Ukl}];
                lbw = [lbw; ulb];
                ubw = [ubw;  uub];
                w0 = [w0; u_init];

                % State at collocation points
                Xklj = cell(1,d);
                Zklj = cell(1,d);
            
                for j=1:d
                    %Differential states
                    Xklj{j} = MX.sym(['X_' num2str(k) '_' num2str(l) '_' num2str(j)], ndiff);
                    w = [w(:)', Xklj(j)];
                    lbw = [lbw; xlb];
                    ubw = [ubw; xub];
                    w0 = [w0; x_init];
                
                    % Algebraic states
                    Zklj{j} = MX.sym(['Z_' num2str(k) '_' num2str(l) '_' num2str(j)], nalg);
                    w = [w(:)', Zklj(j)];
                    lbw = [lbw; zlb];
                    ubw = [ubw; zub];
                    w0 = [w0; z_init];                
                end

                % Loop over collocation points
                Xkl_end = D(1)*Xkl;
        
                for j=1:d
                    % Expression for the state derivative at the collocation point
                    xp = C(1,j+1)*Xkl;
                    zp = tol*ones(nalg,1);
                    for r=1:d
                        xp = xp + C(r+1,j+1)*Xklj{r};
                    end
                              
                    % Append collocation equations
                    [fj, qj] = f(Xklj{j}, Zklj{j}, Ukl, scens{l}');
                    g = [g(:)', {h*fj - [xp; zp]}];
                    lbg = [lbg; zeros(nx,1)];
                    ubg = [ubg; zeros(nx,1)];

                    % Add contribution to the end state
                    Xkl_end = Xkl_end + D(j+1)*Xklj{j};

                    % Add contribution to quadrature function
                    J = J + B(j+1)*qj*h;
                end

                % New NLP variable for state at end of interval
                % Differential states
                Xkl = MX.sym(['X_' num2str(k+1) '_' num2str(l)], ndiff);
                w = [w(:)', {Xkl}];
                lbw = [lbw; xlb];
                ubw = [ubw; xub];
                w0 = [w0; x_init];
                
                % Add inequality constraint
                g = [g(:)', {Ukl(6) + Ukl(4)*rho*cp*(Xkl(4) - scens{l}(2))}];
                lbg = [lbg; Q_demand((ii-1)+k+1)];
                ubg = [ubg; Q_demand((ii-1)+k+1)];

                % Add equality constraint
                g = [g(:)', {Xkl_end - Xkl}];
                lbg = [lbg; zeros(ndiff,1)];
                ubg = [ubg; zeros(ndiff,1)]; 
            end
        end
    
    else
        
        % Start with an empty NLP
        w={};
        w0 = w_stored;
        lbw = [];
        ubw = [];
        J = 0;
        g={};
        lbg = [];
        ubg = [];

        % Apply the first control from the previous solution
        w0(1:ndiff+nu) = [x_init; u_init];
    
        % "Lift" initial conditions
        Xkl = MX.sym('X0', ndiff);
        w = [w(:)', {Xkl}];
        lbw = [lbw; w0(1:ndiff)];
        ubw = [ubw; w0(1:ndiff)];    
    
        % Formulate the NLP
        % For each scenario 
        for l=1:S
            % New NLP variable for the control
            for k=0:N-1
                Ukl = MX.sym(['U_' num2str(k) '_' num2str(l)], nu);
                w = [w(:)', {Ukl}];
                lbw = [lbw; ulb];
                ubw = [ubw; uub];
 
                % State at collocation points
                Xklj = cell(1,d);
                Zklj = cell(1,d);
            
                for j=1:d
                    %Differential states
                    Xklj{j} = MX.sym(['X_' num2str(k) '_' num2str(l) '_' num2str(j)], ndiff);
                    w = [w(:)', Xklj(j)];
                    lbw = [lbw; xlb];
                    ubw = [ubw; xub];
                
                    % Algebraic states
                    Zklj{j} = MX.sym(['Z_' num2str(k) '_' num2str(l) '_' num2str(j)], nalg);
                    w = [w(:)', Zklj(j)];
                    lbw = [lbw; zlb];
                    ubw = [ubw; zub];             
                end

                % Loop over collocation points
                Xkl_end = D(1)*Xkl;
        
                for j=1:d
                    % Expression for the state derivative at the collocation point
                    xp = C(1,j+1)*Xkl;
                    zp = tol*ones(nalg,1);
                    for r=1:d
                        xp = xp + C(r+1,j+1)*Xklj{r};
                    end
                
                    % Append collocation equations
                    [fj, qj] = f(Xklj{j}, Zklj{j}, Ukl, scens{l}');
                    g = [g(:)', {h*fj - [xp; zp]}];
                    lbg = [lbg; zeros(nx,1)];
                    ubg = [ubg; zeros(nx,1)];

                    % Add contribution to the end state
                    Xkl_end = Xkl_end + D(j+1)*Xklj{j};

                    % Add contribution to quadrature function
                    J = J + B(j+1)*qj*h;
                end              
                                      
                % New NLP variable for state at end of interval
                % Differential states
                Xkl = MX.sym(['X_' num2str(k+1) '_' num2str(l)], ndiff);
                w = [w(:)', {Xkl}];
                lbw = [lbw; xlb];
                ubw = [ubw; xub];
                
                % Add inequality constraint
                g = [g(:)', {Ukl(6) + Ukl(4)*rho*cp*(Xkl(4) - scens{l}(2))}];
                lbg = [lbg; Q_demand((ii-1)+k+1)];
                ubg = [ubg; Q_demand((ii-1)+k+1)];

                % Add equality constraint
                g = [g(:)', {Xkl_end - Xkl}];
                lbg = [lbg; zeros(ndiff,1)];
                ubg = [ubg; zeros(ndiff,1)];
            end
           
        end
                
    end
    
    % Non-anticipativity constraints
    U = w(2:2*d+2:end);
    
    for n=1:S-1
        g = [g(:)', {U{N*(n-1)+1} - U{1+N*n}}]; 
        lbg = [lbg; -tol*zeros(nu,1)];
        ubg = [ubg; tol*zeros(nu,1)];
    end
    
    % Create an NLP solver
    opts = struct;
    opts.ipopt.max_iter = maxiter; %5000;
    %opts.ipopt.print_level = 5; %0,3
    opts.print_time = 1; %0,1
    opts.ipopt.tol = tol;
    opts.ipopt.acceptable_tol =100*tol; % optimality convergence tolerance

    prob = struct('f', J, 'x', vertcat(w{:}), 'g', vertcat(g{:}));
    solver = nlpsol('solver', 'ipopt', prob, opts);

    % Solve the NLP
    sol = solver('x0', w0, 'lbx', lbw, 'ubx', ubw,'lbg', lbg, 'ubg', ubg);
    
    if solver.stats.success == 0
       error('Error: Optimal Solution Not Found') 
    end
    
    i_infeasible(ii) = solver.stats.success;
    solver_return{ii} = solver.stats.return_status; 
    w_opt = full(sol.x);
    
    % Store the open loop solution
    w_stored = [w_opt((ndiff+nu)+d*nx+1:end); w_opt(end+1-((ndiff+nu)+d*nx):end)];
    z_init = w_stored((d-1)*ndiff+nu+1:ndiff+nu+nx);

    % Simulator
    x0 = x_init; %z_init];
    u_init = w_opt(ndiff+1:ndiff+nu);
    tsample = T/M;
    tspan = [0 tsample];

    par = [u_init(1:nu-1); T1_m(ii); T2_m(ii); V_hex; V_tank; U_hex; A_hex; rho;... 
        cp; h_s; A_tank; T_s; n];
    Mass = zeros(nx);
    Mass(1:ndiff,1:ndiff) = eye(ndiff);
    options = odeset('RelTol',1e-5,'Stats','off','OutputFcn',@odeplot);
    [t,x] = ode15s(@(t,x) twoPlantModelChen(t,x,par), tspan, x0, options);

    u_opt(ii,:) = w_opt(ndiff+1:ndiff+nu);
    x_m = x(end,1:ndiff);
    x_opt(ii,:) = x_m;
    x_init = transpose(x_m);

    x_init
    u_init
    ii
end

T = T/3600;
tgrid = linspace(0, T, N+1);
clf;

x_opt = [x_0'; x_opt];

% Plot state trajectory
figure(1)
plot(tgrid, x_opt(:,1), '-')
hold on
plot(tgrid, x_opt(:,2), '-')
plot(tgrid, x_opt(:,3), '-')
plot(tgrid, x_opt(:,4), '-')
plot(tgrid, x_opt(:,5), '-')
xlabel('t [hr]')
ylabel('T [\circ C]')
xlim([0 24])
legend('T_{L1}','T_{R1}','T_{L2}','T_{R2}','T_{tank}')
hold off
grid on

% Plot optimal controls
figure(2)
subplot(211)
stairs(tgrid, u_opt(:,1), 'r-.')
hold on
stairs(tgrid, u_opt(:,2), 'g-.')
stairs(tgrid, u_opt(:,3), 'b-.')
stairs(tgrid, u_opt(:,4), 'k-.')
hold off
xlabel('t [hr]')
ylabel('q_{i} [m^3/s]')
xlim([0 24])
legend('q_{L1}','q_{R2}','q_{L2}','q_{R2}')
grid on

subplot(212)
stairs(tgrid, u_opt(:,5), '-.')
hold on
stairs(tgrid, u_opt(:,6), '-.')
stairs(tgrid, Q_demand(1:size(tgrid,2)), 'k*')
hold off
xlabel('t [hr]')
ylabel('Energy supply rate [kW]')
xlim([0 24])
legend('Q_{tank}','Q_M')
grid on

% Plot demand profile
figure(3)
stairs(tgrid, Q_demand(1:size(tgrid,2)), '-')
hold on
plot(tgrid+0.5, Q_demand(1:size(tgrid,2)), 'k--')
plot(tgrid, mean(Q_demand(1:size(tgrid,2)))*ones(size(tgrid,2),1),'r--','LineWidth',1.5)
hold off
xlabel('t [hr]')
ylabel('Energy demand rate [kW]')
ylim([0 12000])
grid on

% Plot the noisy inputs
figure(4)
subplot(311)
plot(tgrid, T1_m, 'r*')
ylabel('T_{1} [\circ C]')
subplot(312)
plot(tgrid, T2_m, 'bx')
ylabel('T_{2} [\circ C]')
subplot(313)
plot(tgrid, T2_m, 'gd')
ylabel('T_{surr} [\circ C]')
xlabel('t [hr]')