function dxdt = twoPlantMdlChen2(~,x,p)
% Model for an energy storage system with two plants
% a heat supplier and consumer, a storage tank and external heating.
%------------------------------------------------------------------
% Description of the states:
%------------------------------------------------------------------
T_L1 = x(1); 
T_R1 = x(2);
T_L2 = x(3);
T_R2 = x(4);
T_tank = x(5);
%------------------------------------------------------------------
% Reassignment of inputs and disturbances
% Manipulated or fixed input variables u's
q_L1 = p(1);
q_R1 = p(2);
q_L2 = p(3);
q_R2 = p(4);
Q_M = p(5);
Q_dump = p(6);
Q_tank = p(7);

% Distubances
T1 = p(8);
T2 = p(9);

% Design and physical parameters
V_hex = p(10);
V_tank = p(11);
U_hex = p(12);
A_hex = p(13);
rho = p(14);
cp = p(15);
h_s = p(16);
A_tank = p(17);
T_s = p(18);
n = p(19);
Q_supply = p(20);
Q_demand = p(21);


%% ODEs
% dxdt = zeros(5,1);

dxdt =  [Q_supply - rho*cp*q_L1*(T1-T_L1) - Q_dump;

         (1/V_hex)*(q_R1*(T_tank-T_R1) + (U_hex*A_hex/(rho*cp))*(0.5*((abs(T1-T_R1))^n + (abs(T_L1-T_tank))^n))^(1/n));

         (1/V_hex)*(q_L2*(T_tank-T_L2) - (U_hex*A_hex/(rho*cp))*(0.5*((abs(T_L2-T2))^n + (abs(T_tank-T_R2))^n))^(1/n));

         Q_demand - rho*cp*q_R2*(T_R2-T2) - Q_M;

         (1/V_tank)*(q_R1*(T_R1-T_tank) + q_L2*(T_L2-T_tank) + (Q_tank-h_s*A_tank*(T_tank-T_s))/(rho*cp))];
       
         
end

