function dxdt = twoPlantModelChen(~,x,p)
% Model for an energy storage system with two plants
% a heat supplier and consumer, a storage tank and external heating.
%------------------------------------------------------------------
% Description of the states:
%------------------------------------------------------------------
T_L1 = x(1); 
T_R1 = x(2);
T_L2 = x(3);
T_R2 = x(4);
T_tank = x(5);
%------------------------------------------------------------------
% Reassignment of inputs and disturbances
% Manipulated or fixed input variables u's
q_L1 = p(1);
q_R1 = p(2);
q_L2 = p(3);
q_R2 = p(4);
Q_tank = p(5);

%% Distubances
T1 = p(6);
T2 = p(7);

%% Design and physical parameters
V_hex = p(8);
V_tank = p(9);
U_hex = p(10);
A_hex = p(11);
rho = p(12);
cp = p(13);
h_s = p(14);
A_tank = p(15);
T_s = p(16);
n = p(17);

%% ODEs

dxdt =  [(1/V_hex)*(q_L1*(T1-T_L1) - (U_hex*A_hex/(rho*cp))*...
        (0.5*((abs(T1-T_R1))^n + (abs(T_L1-T_tank))^n))^(1/n));

         (1/V_hex)*(q_R1*(T_tank-T_R1) + (U_hex*A_hex/(rho*cp))*...
         (0.5*((abs(T1-T_R1))^n + (abs(T_L1-T_tank))^n))^(1/n));

         (1/V_hex)*(q_L2*(T_tank-T_L2) - (U_hex*A_hex/(rho*cp))*...
         (0.5*((abs(T_L2-T2))^n + (abs(T_tank-T_R2))^n))^(1/n));

         (1/V_hex)*(q_R2*(T2-T_R2) + (U_hex*A_hex/(rho*cp))*...
         (0.5*((abs(T_L2-T2))^n + (abs(T_tank-T_R2))^n))^(1/n));

         (1/V_tank)*(q_R1*(T_R1-T_tank) + q_L2*(T_L2-T_tank) ...
         + (Q_tank-h_s*A_tank*(T_tank-T_s))/(rho*cp))];

end

