function dxdt = twoPlantModelDirect(~,x,p)
% Model for direct heat exchange system between two plants
% a heat supplier and consumer external heating.
%========================================================================
% Author: Zawadi Mdoe
% Date: May 2019
%========================================================================
%% Description of the states:
T_L1 = x(1); 
T_R1 = x(2);

%=======================================================================
%% Assignment of inputs and disturbances
% Input variables u's
q_L1 = p(1);
q_R1 = p(2);

% Distubances d's
T1 = p(3);
T2 = p(4);

% Design and physical parameters
V_hex = p(5);
U_hex = p(6);
A_hex = p(7);
rho = p(8);
cp = p(9);
n = p(10);

%% ODEs
dxdt =  [(1/V_hex)*(q_L1*(T1-T_L1) - (U_hex*A_hex/(rho*cp))*...
        (0.5*((abs(T1 - T_R1))^n + (abs(T_L1 - T2))^n))^(1/n));

         (1/V_hex)*(q_R1*(T2-T_R1) + (U_hex*A_hex/(rho*cp))*...
         (0.5*((abs(T1 - T_R1))^n + (abs(T_L1 - T2))^n))^(1/n))];     
        
end

