function dxdt = twoPlantModelDirect2(~,x,p)
% Model for direct heat exchange system between two plants
% a heat supplier and consumer external heating.
%========================================================================
% Author: Zawadi Mdoe
% Date: May 2019
%========================================================================
% Description of the states:
T_L1 = x(1); 
T_R1 = x(2);
Tr = x(3);
T1 = x(4);
%=======================================================================
%% Assignment of inputs and disturbances
% Input variables u's
q_L1 = p(1);
q_R1 = p(2);
Q_dump = p(3);

% Distubances d's
T2 = p(4);

% Design and physical parameters
Ve = p(5);
V_hex = p(6);
U_hex = p(7);
A_hex = p(8);
rho = p(9);
cp = p(10);
n = p(11);
Q_supply = p(12);


%% ODEs
dxdt =  [(1/V_hex)*(q_L1*(T1-T_L1) - (U_hex*A_hex/(rho*cp))*...
        (0.5*((abs(T1 - T_R1))^n + (abs(T_L1 - T2))^n))^(1/n));

         (1/V_hex)*(q_R1*(T2-T_R1) + (U_hex*A_hex/(rho*cp))*...
         (0.5*((abs(T1 - T_R1))^n + (abs(T_L1 - T2))^n))^(1/n));
     
        (1/Ve)*(q_L1*(Tr - T_L1) - Q_dump/(rho*cp));
     
        (1/Ve)*(q_L1*(T1 - Tr) + Q_supply/(rho*cp))];
        

end

