function dxdt = twoPlantModel(~,x,p)
% Model for an energy storage system with two plants
% a heat supplier and consumer, a storage tank and external heating.
%------------------------------------------------------------------
% Description of the states:
%------------------------------------------------------------------
% T_L1 = x(1) 
% T_R1 = x(2)
% T_L2 = x(3)
% T_R2 = x(4)
% T_tank = x(5)
std = 2.31;
%------------------------------------------------------------------
% Reassignment of inputs and disturbances
% Manipulated or fixed input variables u's
q_L1 = p(1);
q_R1 = p(2);
q_L2 = p(3);
q_R2 = p(4);
Q = p(5);

% Distubances
T1 = p(6)+std*randn(1);
T2 = p(7)+std*randn(1);
T_s = p(16)+std*randn(1);

% Design and physical parameters
V_hex = p(8);
V_tank = p(9);
U_hex = p(10);
A_hex = p(11);
rho = p(12);
cp = p(13);
h_s = p(14);
A_tank = p(15);
n = p(17);

%% ODEs
dxdt = zeros(5,1);

dxdt =  [(1/V_hex)*(q_L1*(T1-x(1)) - sign(T1-x(5))*(U_hex*A_hex*0.5^(1/n)/(rho*cp))*((abs(T1-x(2)))^n + (abs(x(1)-x(5)))^n)^(1/n));

         (1/V_hex)*(q_R1*(x(5)-x(2)) + sign(T1-x(5))*(U_hex*A_hex*0.5^(1/n)/(rho*cp))*((abs(T1-x(2)))^n + (abs(x(1)-x(5)))^n)^(1/n));

         (1/V_hex)*(q_L2*(x(5)-x(3)) - sign(x(5)-T2)*(U_hex*A_hex*0.5^(1/n)/(rho*cp))*((abs(x(3)-T2))^n + (abs(x(5)-x(4)))^n)^(1/n));

         (1/V_hex)*(q_R2*(T2-x(4)) + sign(x(5)-T2)*(U_hex*A_hex*0.5^(1/n)/(rho*cp))*((abs(x(3)-T2))^n + (abs(x(5)-x(4)))^n)^(1/n));

         (1/V_tank)*(q_R1*(x(2)-x(5)) + q_L2*(x(3)-x(5)) + (Q-h_s*A_tank*(x(5)-T_s))/(rho*cp))];

end

