clear;
close all;
clc;

load NMPCwithstorage.mat
load NMPCnostorage.mat

J_direct = nostorageNMPC.Cost;
J_storage = storageNMPC.Cost;

T = 24;
N = 24;
tgrid = linspace(0, T, N+1);
clf;

%% Plotting
set(0,'DefaultTextFontName','Times',...
'DefaultTextFontSize',22,...
'DefaultAxesFontName','Times',...
'DefaultAxesFontSize',20,...
'DefaultLineLineWidth',1.5,...
'DefaultLineMarkerSize',7.75,...
'DefaultStairLineWidth',1.5);
set(findall(gcf,'Type','text'),'FontSize',18,'Interpreter','latex');
set(gcf,'color','white');

% Compare costs
figure(1)
subplot(211)
stairs(tgrid, J_direct, 'b')
hold on
stairs(tgrid, J_storage, 'r')
hold off
grid on
legend('no storage','with storage')
ylabel('Cost [units/hr]')
xlabel('time [hr]')
subplot(212)
stairs(tgrid, [nostorageNMPC.OptimalInputs(2,:), NaN],'b')
hold on
stairs(tgrid, [storageNMPC.OptimalInputs(3,:), NaN],'r')
hold off
grid on
ylabel('External heating [kW]')
xlabel('time [hr]')


figure(2)
stairs(tgrid, [nostorageNMPC.OptimalInputs(2,:), NaN],'b')
hold on
stairs(tgrid, [storageNMPC.OptimalInputs(3,:), NaN],'r')
hold off
grid on
legend('no storage','with storage')
ylabel('External heating [kW]')
xlabel('time [hr]')


figure(3)
subplot(211)
stairs(tgrid, 50*ones(size(tgrid)), 'r-')
hold on
stairs(tgrid, [storageNMPC.OptimalInputs(1,:), nan], 'b-')
hold off
ylabel('q_{i} [m^3/s]')

legend('q_{L2}','q_{R2}')
grid on

subplot(212)
stairs(tgrid, [storageNMPC.OptimalInputs(2,:), nan], 'b-')
hold on
stairs(tgrid, [storageNMPC.OptimalInputs(3,:), nan], 'r-')
stairs(tgrid, [storageNMPC.OptimalInputs(4,:), nan], 'g-')
% stairs(tgrid, Q_demand(1:size(tgrid,2)), 'k*')
hold off
xlabel('time [hr]')
ylabel('Energy supply rate [kW]')
legend('Q_{tank}','Q_M','Q_{dump}')
grid on

figure(4)
plot(tgrid, [storageNMPC.OptimalStates(1,:), NaN], 'r-')
hold on
plot(tgrid, [storageNMPC.OptimalStates(5,:), NaN], 'b-')
xlabel('time [hr]')
ylabel('Temperature [\circ C]')
legend('T_{L1}','T_{tank}')
hold off
grid on