function dxdt = EnergyStorage(~,x,p)
% Model for an energy storage system with two plants
% a heat supplier and consumer, a storage tank and external heating.
%------------------------------------------------------------------
% Description of the states:
%------------------------------------------------------------------
T_L2 = x(1); 
T_R2 = x(2); 
T_tank = x(3);     
x4 = x(4); 
x5 = x(5);

%------------------------------------------------------------------
% Reassignment of inputs and disturbances
% Manipulated or fixed input variables u's
q_L1 = p(1);
T_L1 = p(2);
q_L2 = p(3);
q_R2 = p(4);
Q = p(5);

% Disturbances
T1 = p(6);
T2 = p(7);

% Design and physical parameters
V_hex = p(8);
V_tank = p(9);
U_hex = p(10);
A_hex = p(11);
rho = p(12);
cp = p(13);
h_s = p(14);
A_tank = p(15);
T_s = p(16);
n = p(17);

%% ODEs
dxdt = [(q_L2*(T_tank-T_L2) - (U_hex*A_hex/(rho*cp))*(0.5*(x4+x5))^(1/n))/V_hex;
    
        (q_R2*(T2-T_R2) + (U_hex*A_hex/(rho*cp))*(0.5*(x4+x5))^(1/n))/V_hex;        
        
        (q_L1*(T1-T_L1)-q_L2*(T_tank-T_L2) + (Q-h_s*A_tank*(T_tank-T_s))/rho*cp)/V_tank;
        
        x4^(1/n) - T_L2 + T2;
        
        x5^(1/n) - T_tank + T_R2];
    
end

