%Model for an energy storage system consiting of two plants
%a heat supplier and consumer, a storage tank and external heating.

clear all;
clc;

%% Declaration of model parameters
V_hex = 0.5;        %Heat exchanger volume (shell or tube side) [m^3]
V_tank = 100;
U_hex = 0.320;            %Overall heat transfer coefficient [kW/m^2K]
A_hex = 300;         %Heat transfer area [m^2]
rho= 1000;           %Density [kg/m^3]
cp = 4.186;            %specific heat capacity of fluid [kJ/kgK]
h_s = 0.050;        %heat loss coefficient [kW/m^2K]
n = 1/3;%0.3275;
A_tank =1000;

%disturbances
T1 = 95;
T2 = 20;
T_s = 15;

%Inputs
q_L1 = 0.5; 
q_R1 = 0.5;
q_L2 = 0.5;
q_R2 = 0.5;
Q = 5e3;

p = [q_L1 q_R1 q_L2 q_R2 Q T1 T2 V_hex V_tank U_hex A_hex rho cp h_s A_tank T_s n];

%% Declaration of variables and their nominal values
x0 = [80 60 45 30 50];
tspan = [0 3600];
N = 24;
nx = 5;
x_f = zeros(N,nx);

for i=1:N
    
    % solving the ode using ode45
    options = odeset('RelTol',1e-5,'Stats','off','OutputFcn',@odeplot);
    [t,x] = ode15s(@(t,x) twoPlantModelChen(t,x,p),tspan,x0,options);
    
    tspan = tspan + 3600;
    x_f(i,:) = x(end,:);
    x0 = x(end,:);
    %t = t/3600;

end
% %% Plotting results
% figure(1)
% subplot(321)
% plot(t,x(:,1),'r-')
% %xlabel('time (s)')
% ylabel('T_{L1} [\circC]')
% %xlim([0 60])
% %ylim([30 100])
% grid on
% 
% subplot(322)
% plot(t,x(:,2),'r-')
% %xlabel('time (s)')
% ylabel('T_{R1} [\circC]')
% %ylim([30 100])
% grid on
% 
% subplot(323)
% plot(t,x(:,3),'b-')
% %ylim([10 50])
% %xlabel('time (s)')
% ylabel('T_{L2} [\circC]')
% grid on
% 
% subplot(324)
% plot(t,x(:,4),'b-')
% %ylim([10 50])
% xlabel('time [h]')
% ylabel('T_{R2} [\circC]')
% grid on
% 
% subplot(325)
% plot(t,x(:,5),'g-')
% xlabel('time [h]')
% ylabel('T_{tank} [\circC]')
% grid on
% 
% x(end,:)
% % legend('T_{L1}','T_{R1}','T_{L2}','T_{R2}','T_{tank}')
 