% An implementation of direct collocation to open loop 
% dynamic optimisation of a two plant 
% Energy Storage System using CasADi
%% VARIABLE DEMAND!!!

% Zawadi Mdoe, 2018
% ============================================================================
clear;
clc;

addpath('C:\Users\DELL\Desktop\Matlab\casadi-windows-matlabR2016a-v3.4.5')
import casadi.*

% run Parameters_EnergyStorageCase3.m
run Parameters_scendae32.m
% load uncertaintyparametersdata_uniform.mat
% load Expecteddemand.mat
% load CaliforniaDemand.mat
% load CaliforniaDemand2.mat
% load CaliforniaDemand4.mat
% load NOdemand.mat

deltaQ = 0;
Q_demand = [1500*ones(12,1); 3500*ones(12,1)];
Q_supply = rho*cp*q_L1*(T1-T_L1req)*ones(24,1);

% Degree of interpolating polynomial
d = 3;

% Get collocation points
tau_root = [0 collocation_points(d, 'radau')]; %can be 'legendre'

% Coefficients of the collocation equation
C = zeros(d+1,d+1);

% Coefficients of the continuity equation
D = zeros(d+1, 1);

% Coefficients of the quadrature function
B = zeros(d+1, 1);

% Construct polynomial basis
for j=1:d+1
  % Construct Lagrange polynomials to get the polynomial basis 
  % at the collocation point
  coeff = 1;
  for r=1:d+1
    if r ~= j
      coeff = conv(coeff, [1, -tau_root(r)]);
      coeff = coeff / (tau_root(j)-tau_root(r));
    end
  end
  % Evaluate the polynomial at the final time to get the 
  % coefficients of the continuity equation
  D(j) = polyval(coeff, 1.0);

  % Evaluate the time derivative of the polynomial at all collocation 
  % points to get the coefficients of the continuity equation
  pder = polyder(coeff);
  for r=1:d+1
    C(j,r) = polyval(pder, tau_root(r));
  end

  % Evaluate the integral of the polynomial to get the coefficients 
  % of the quadrature function
  pint = polyint(coeff);
  B(j) = polyval(pint, 1.0);
end

% Time horizon
T = 24*60*60;
ndiff = 5;                 %number of differential states
nalg = 4;                 %number of algebraic states
nu = 3;                 %number of controls
nx = nalg + ndiff;           %total number of states

zub = Inf*ones(nalg,1);
zlb = -Inf*ones(nalg,1);

run SSOptwtankheating2.m
x_0 = x_init;

% Declare model variables
x1 = SX.sym('x1');
x2 = SX.sym('x2');
x3 = SX.sym('x3');
x4 = SX.sym('x4');
x5 = SX.sym('x5');
x6 = SX.sym('x6');
x7 = SX.sym('x7');
x8 = SX.sym('x8');
x9 = SX.sym('x9');
x = [x1; x2; x3; x4; x5];
z = [x6; x7; x8; x9];
u1 = SX.sym('u1');
u2 = SX.sym('u2');
u3 = SX.sym('u3');
u = [u1; u2; u3];

eps = MX.sym('eps', ndiff);

% Model equations

xdot =  [(1/V_hex)*(q_L1*(T1-x1) - h_dot*(0.5*(x6 + x7))^(1/n));

        (1/V_hex)*(q_R1*(x5-x2) + h_dot*(0.5*(x6 + x7))^(1/n));

        (1/V_hex)*(u1*(x5-x3) - h_dot*(0.5*(x8 + x9))^(1/n));

        (1/V_hex)*(u1*(T2-x4) + h_dot*(0.5*(x8 + x9))^(1/n));
        
        (1/V_tank)*(q_R1*(x2-x5) + u1*(x3-x5) + (u2/(rho*cp)-h_t_dot*(x5-T_s)));
        
        T1-x2-x6^(1/n);
        
        x1-x5-x7^(1/n);
        
        x3-T2-x8^(1/n);
        
        x5-x4-x9^(1/n)];

% Objective term
L = Pm*u3 + Pt*u2 + Pu*u1.^2;

% Continuous time dynamics
f = Function('f', {x, z, u}, {xdot, L});

% Control discretization
N = 24; % number of control intervals
M = 24; % number of MPC loops
h = T/N;
period = N;
% Q_demand = zeros(N+M,1);

% Variable counting
NXD = N*ndiff*(d+1);    %Total Number of differential state variables 
NXA = N*nalg*d;         %Total Number of algebraic state variables
NU = N*nu;              %Total Number of input variables
NXF = ndiff;            %Total Number of end point variables (differential only)
NV = NXD + NXA + NU + NXF; %Total number of NLP variables

%% Prepare output variables
x_opt = zeros(ndiff,M);
u_opt = zeros(nu,M);
i_infeasible = zeros(M,1);
solver_return = {};
w_stored = zeros(NV,1);

%% Predicted Demand
Q_demand = [Q_demand; Q_demand];

%% MPC loop
for i=1:M
    if i==1  
        % Start with an empty NLP
        w={};
        w0 = [];
        lbw = [];
        ubw = [];
        J = 0;
        g={};
        lbg = [];
        ubg = [];

        % "Lift" initial conditions
        % Differential states
        Xk = MX.sym('X0', ndiff);
        w = [w(:)', {Xk}];
        lbw = [lbw; x_init];
        ubw = [ubw; x_init];
        w0 = [w0; x_init];
    
        % Formulate the NLP
        for k=0:N-1
            % New NLP variable for the control
            Uk = MX.sym(['U_' num2str(k)],nu); 
            w = [w(:)', {Uk}];
            lbw = [lbw; ulb];
            ubw = [ubw; uub];
            w0 = [w0; u_init];
  
            % State at collocation points
            Xkj = cell(1,d);
            Zkj = cell(1,d);
            
            for j=1:d
                % Differential states
                Xkj{j} = MX.sym(['X_' num2str(k) '_' num2str(j)],ndiff);
                w = [w(:)', Xkj(j)];
                lbw = [lbw; xlb];
                ubw = [ubw; xub];
                w0 = [w0; x_init];
            
                % Algebraic states
                Zkj{j} = MX.sym(['Z_' num2str(k) '_' num2str(j)],nalg);
                w = [w(:)', Zkj(j)];
                lbw = [lbw; zlb];
                ubw = [ubw; zub];
                w0 = [w0; z_init];           
            end

            % Loop over collocation points
            Xk_end = D(1)*Xk;
       
            for j=1:d
                % Expression for the state derivative at the collocation point
                xp = C(1,j+1)*Xk;
                zp = zeros(nalg,1);
                for r=1:d
                    xp = xp + C(r+1,j+1)*Xkj{r};              
                end
        
                % Append collocation equations
                [fj, qj] = f(Xkj{j}, Zkj{j}, Uk);
                g = [g(:)', {h*fj - [xp;zp]}];
                lbg = [lbg; zeros(nx,1)];
                ubg = [ubg; zeros(nx,1)];

                % Add contribution to the end state
                Xk_end = Xk_end + D(j+1)*Xkj{j};
                        
                % Add contribution to quadrature function
                J = J + B(j+1)*qj*h;
            end

            % New NLP variable for state at end of interval
            % Differential states
            Xk = MX.sym(['X_' num2str(k+1)], ndiff);
            w = [w(:)', {Xk}];
            lbw = [lbw; xlb];
            ubw = [ubw; xub];
            w0 = [w0; x_init];
            
            % Add inequality constraint
            g = [g(:)', {Uk(3) + Uk(1)*rho*cp*(Xk(4)-T2)}];
            lbg = [lbg; Q_demand((i-1)+k+1)];
            ubg = [ubg; Q_demand((i-1)+k+1)];
    
            % Add equality constraint
            g = [g(:)', {Xk_end-Xk}];
            lbg = [lbg; zeros(ndiff,1)];
            ubg = [ubg; zeros(ndiff,1)];
        end
        
    else
        % Start with an empty NLP
        w={};
        w0 = w_stored;
        lbw = [];
        ubw = [];
        J = 0;
        g={};
        lbg = [];
        ubg = [];
        
        % Apply the first control from the previous solution
        w0(1:ndiff+nu) = [x_init; u_init];
        
        % "Lift" initial conditions
        % Differential states
        Xk = MX.sym('X0', ndiff);
        w = [w(:)', {Xk}];
        lbw = [lbw; w0(1:ndiff)];
        ubw = [ubw; w0(1:ndiff)];
            
        % Formulate the NLP
        for k=0:N-1
            % New NLP variable for the control
            Uk = MX.sym(['U_' num2str(k)],nu); 
            w = [w(:)', {Uk}];
            lbw = [lbw; ulb];
            ubw = [ubw; uub];
                          
            % State at collocation points
            Xkj = {};
            Zkj = {};
            for j=1:d
                % Differential states
                Xkj{j} = MX.sym(['X_' num2str(k) '_' num2str(j)],ndiff);
                w = [w(:)', Xkj(j)];
                lbw = [lbw; xlb];
                ubw = [ubw; xub];
                                            
                % Algebraic states
                Zkj{j} = MX.sym(['Z_' num2str(k) '_' num2str(j)],nalg);
                w = [w(:)', Zkj(j)];
                lbw = [lbw; zlb];
                ubw = [ubw; zub];       
            end

            % Loop over collocation points
            Xk_end = D(1)*Xk;
            for j=1:d
                % Expression for the state derivative at the collocation point
                xp = C(1,j+1)*Xk;
                zp = zeros(nalg,1);
                for r=1:d
                    xp = xp + C(r+1,j+1)*Xkj{r};              
                end
        
                % Append collocation equations
                [fj, qj] = f(Xkj{j},Zkj{j},Uk);
                g = [g(:)', {h*fj - [xp;zp]}];
                lbg = [lbg; zeros(nx,1)];
                ubg = [ubg; zeros(nx,1)];

                % Add contribution to the end state
                Xk_end = Xk_end + D(j+1)*Xkj{j};
                        
                % Add contribution to quadrature function
                J = J + B(j+1)*qj*h;
            end

            % New NLP variable for state at end of interval
            % Only for differential states
            Xk = MX.sym(['X_' num2str(k+1)], ndiff);
            w = [w(:)', {Xk}];
            lbw = [lbw; xlb];
            ubw = [ubw; xub];  
       
            % Add inequality constraint
            g = [g(:)', {Uk(3) + Uk(1)*rho*cp*(Xk(4)-T2)}];
            lbg = [lbg; Q_demand((i-1)+k+1)];
            ubg = [ubg; Q_demand((i-1)+k+1)];

            % Add equality constraint (only differential states)
            g = [g(:)', {Xk_end-Xk}];
            lbg = [lbg; zeros(ndiff,1)];
            ubg = [ubg; zeros(ndiff,1)];
        end        
    end
    
    % Create an NLP solver
    opts = struct;
    opts.ipopt.max_iter = maxiter; %5000;
    opts.ipopt.print_level = 3; %0,3
    opts.print_time = 1; %0,1
    opts.ipopt.tol = tol;
    opts.ipopt.acceptable_tol =100*tol; % optimality convergence tolerance

    prob = struct('f', J, 'x', vertcat(w{:}), 'g', vertcat(g{:}));
    solver = nlpsol('solver', 'ipopt', prob, opts);

    % Solve the NLP
    sol = solver('x0', w0, 'lbx', lbw, 'ubx', ubw,'lbg', lbg, 'ubg', ubg);
    
    if solver.stats.success == 0
       error('Error: Optimal Solution Not Found') 
    end
    
    i_infeasible(i) = solver.stats.success;
    solver_return{i} = solver.stats.return_status; 
    w_opt = full(sol.x);

    % Store the open loop solution
    w_stored = [w_opt((ndiff+nu)+d*nx+1:end); w_opt(end+1-((ndiff+nu)+d*nx):end)];
    
    % Simulator
    x0 = x_init;
    tspan = [0 3600];
    p = [q_L1; q_R1; w_opt(ndiff+1); w_opt(ndiff+1:ndiff+nu-1); 90; 25; V_hex; V_tank; U_hex; A_hex; rho;... 
        cp; h_s; A_tank; T_s; n];
    options = odeset('RelTol', 1e-8,'Stats','off','OutputFcn', @odeplot);
    [t,x] = ode15s(@(t,x) twoPlantModelChen(t,x,p), tspan, x0, options);

    u_opt(:,i) = w_opt(ndiff+1:ndiff+nu);
    u_init = w_opt(ndiff+1:ndiff+nu);
    x_m = x(end,:);
    x_opt(:,i) = x_m;
    x_init = x_m';

    x_init
    u_init
    i
    
    if i==1
        w_ol = w_opt;
    end
    
end

% Cost = Pm*u_opt(3,:) + Pt*u_opt(2,:);
Cost = 0;

% Store data
storageNMPC = struct('Demand',Q_demand,'OptimalStates',...
                    x_opt,'OptimalInputs',u_opt,'Measurement',...
                    x_opt,'Cost',Cost);

%% Plot results
T = T/3600;
tgrid = linspace(0, T, N+1);
clf;

x_opt = [x_0, x_opt];

set(0,'DefaultTextFontName','Times',...
'DefaultTextFontSize',15,...
'DefaultAxesFontName','Times',...
'DefaultAxesFontSize',15,...
'DefaultLineLineWidth',1.5,...
'DefaultLineMarkerSize',7.75,...
'DefaultStairLineWidth',1.5);
set(findall(gcf,'Type','text'),'FontSize',15,'Interpreter','latex');
set(gcf,'color','white');

% Plot state trajectory
figure(1)
subplot(311)
plot(tgrid, x_opt(1,:), 'b-')
hold on
% plot(tgrid, x_opt(2,:), '-')
% plot(tgrid, x_opt(3,:), '-')
% plot(tgrid, x_opt(4,:), '-')
plot(tgrid, x_opt(5,:), 'k-')
plot(tgrid, T_L1req*ones(25,1), 'm--','LineWidth',2)
% xlabel('time [hr]','Interpreter','latex')
ylabel('T [$^{\circ}$C]','Interpreter','latex')
xlim([0 24])
legend('$T_{L1}$','$T_{tank}$','Interpreter','latex')
hold off
grid on


% Plot optimal controls
subplot(312)
stairs(tgrid, [u_opt(1,:), nan], 'b-')
hold on
% stairs(tgrid, [u_opt(1,:), nan], 'b-')
hold off
% xlabel('t [hr]')
ylabel('$q_{R2}$ [$m^3$/s]','Interpreter','latex')
xlim([0 24])
% legend('$q_{R2}$','Interpreter','latex')
grid on

subplot(313)
stairs(tgrid, [u_opt(2,:), nan], 'b-')
hold on
stairs(tgrid, [u_opt(3,:), nan], 'r-')
% stairs(tgrid, Q_demand(1:size(tgrid,2)), 'k*')
hold off
xlabel('time [hr]','Interpreter','latex')
ylabel('Power [kW]','Interpreter','latex')
xlim([0 24])
legend('$Q_{tank}$','$Q_M$','Interpreter','latex')
grid on

% Plot demand profile
% figure(3)
% stairs(tgrid, Q_demand(1:size(tgrid,2)), '-')
% hold on
% plot(tgrid+0.5, Q_demand(1:size(tgrid,2)), 'k--')
% plot(tgrid, mean(Q_demand(1:size(tgrid,2)))*ones(size(tgrid,2),1),'r--','LineWidth',1.5)
% hold off
% xlabel('t [hr]')
% ylabel('Energy demand rate [kW]')
% ylim([0 12000])
% grid on