% Steady-state optimisation

% run Parameters_TES.m

zub = Inf*ones(nalg,1);
zlb = -Inf*ones(nalg,1);

addpath('C:\Users\DELL\Desktop\Matlab\casadi-windows-matlabR2016a-v3.4.5')
import casadi.*

% Time horizon
T = 24*60*60;
ndiff = 5;                 %number of differential states
nalg = 4;                 %number of algebraic states
nu = 3;                 %number of controls
nx = nalg + ndiff;           %total number of states

% Declare model variables
x1 = SX.sym('x1');
x2 = SX.sym('x2');
x3 = SX.sym('x3');
x4 = SX.sym('x4');
x5 = SX.sym('x5');
x6 = SX.sym('x6');
x7 = SX.sym('x7');
x8 = SX.sym('x8');
x9 = SX.sym('x9');
x = [x1; x2; x3; x4; x5];
z = [x6; x7; x8; x9];
u1 = SX.sym('u1');
u2 = SX.sym('u2');
u3 = SX.sym('u3');
u = [u1; u2; u3];


xdot =  [(1/V_hex)*(q_L1*(T1-x1) - h_dot*(0.5*(x6 + x7))^(1/n));

        (1/V_hex)*(q_R1*(x5-x2) + h_dot*(0.5*(x6 + x7))^(1/n));

        (1/V_hex)*(q_L2*(x5-x3) - h_dot*(0.5*(x8 + x9))^(1/n));

        (1/V_hex)*(u1*(T2-x4) + h_dot*(0.5*(x8 + x9))^(1/n));
        
        (1/V_tank)*(q_R1*(x2-x5) + q_L2*(x3-x5) - h_t_dot*(x5-T_s));
        
        T1-x2-x6^(1/n);
        
        x1-x5-x7^(1/n);
        
        x3-T2-x8^(1/n);
        
        x5-x4-x9^(1/n)];
    
% Objective term
L = Pm*u2;

% Continuous timem dynamics
f = Function('f', {x, z, u}, {xdot, L});

% Control discretization
N = 24; % number of control intervals
M = 24; % number of MPC loops
h = T/N;
period = N;
% Q_demand = zeros(N+M,1);

% Variable counting
NXD = N*ndiff*(d+1);    %Total Number of differential state variables 
NXA = N*nalg*d;         %Total Number of algebraic state variables
NU = N*nu;              %Total Number of input variables
NXF = ndiff;            %Total Number of end point variables (differential only)
NV = NXD + NXA + NU + NXF; %Total number of NLP variables


% Start with an empty NLP
w={};
w0 = [];
lbw = [];
ubw = [];
J = 0;
g={};
lbg = [];
ubg = [];

% "Lift" initial conditions
% Differential states
Xk = MX.sym('X0', ndiff);
w = [w(:)', {Xk}];
lbw = [lbw; x_init];
ubw = [ubw; x_init];
w0 = [w0; x_init];
    
% Formulate the NLP
for k=0:1
    % New NLP variable for the control
    Uk = MX.sym(['U_' num2str(k)], nu); 
    w = [w(:)', {Uk}];
    lbw = [lbw; ulb];
    ubw = [ubw; uub];
    w0 = [w0; u_init];
  
    % State at collocation points
    Xkj = cell(1,d);
    Zkj = cell(1,d);
            
    for j=1:d
        % Differential states
        Xkj{j} = MX.sym(['X_' num2str(k) '_' num2str(j)],ndiff);
        w = [w(:)', Xkj(j)];
        lbw = [lbw; xlb];
        ubw = [ubw; xub];
        w0 = [w0; x_init];
            
        % Algebraic states
        Zkj{j} = MX.sym(['Z_' num2str(k) '_' num2str(j)],nalg);
        w = [w(:)', Zkj(j)];
        lbw = [lbw; zlb];
        ubw = [ubw; zub];
        w0 = [w0; z_init];           
    end

    % Loop over collocation points
    Xk_end = D(1)*Xk;
       
    for j=1:d
        % Expression for the state derivative at the collocation point
        xp = C(1,j+1)*Xk;
        zp = zeros(nalg,1);
        
        for r=1:d
            xp = xp + C(r+1,j+1)*Xkj{r};              
        end
        
        xp = zeros(ndiff,1);
        
        % Append collocation equations
        [fj, qj] = f(Xkj{j}, Zkj{j}, Uk);
        g = [g(:)', {h*fj - [xp; zp]}];
        lbg = [lbg; zeros(nx,1)];
        ubg = [ubg; zeros(nx,1)];

        % Add contribution to the end state
        Xk_end = Xk_end + D(j+1)*Xkj{j};
                        
        % Add contribution to quadrature function
        J = J + B(j+1)*qj*h;
    end

    % New NLP variable for state at end of interval
    % Differential states
    Xk = MX.sym(['X_' num2str(k+1)], ndiff);
    w = [w(:)', {Xk}];
    lbw = [lbw; xlb];
    ubw = [ubw; xub];
    w0 = [w0; x_init];
           
    % Add inequality constraint
    g = [g(:)', {Uk(2) + Uk(1)*rho*cp*(Xk(4)-T2)}];
    lbg = [lbg; Q_supply(1)];
    ubg = [ubg; Q_supply(1)];
          
    g = [g(:)', {Uk(3) + q_L1*rho*cp*(T1-Xk(1))}];
    lbg = [lbg; Q_supply(1)];
    ubg = [ubg; Q_supply(1)];
    
    % Add equality constraint
    g = [g(:)', {Xk_end-Xk}];
    lbg = [lbg; zeros(ndiff,1)];
    ubg = [ubg; zeros(ndiff,1)];

end

% Create an NLP solver
opts = struct;
opts.ipopt.max_iter = maxiter; %5000;
opts.ipopt.print_level = 3; %0,3
opts.print_time = 1; %0,1
opts.ipopt.tol = tol;
opts.ipopt.acceptable_tol =100*tol; % optimality convergence tolerance

prob = struct('f', J, 'x', vertcat(w{:}), 'g', vertcat(g{:}));
solver = nlpsol('solver', 'ipopt', prob, opts);

% Solve the NLP
sol = solver('x0', w0, 'lbx', lbw, 'ubx', ubw,'lbg', lbg, 'ubg', ubg);
    
if solver.stats.success == 0
    error('Error: Optimal Solution Not Found') 
end

w_opt = full(sol.x);

x1_optss = w_opt(1:(ndiff+nu)+nx*d:end);
x2_optss = w_opt(2:(ndiff+nu)+nx*d:end);
x3_optss = w_opt(3:(ndiff+nu)+nx*d:end);
x4_optss = w_opt(4:(ndiff+nu)+nx*d:end);
x5_optss = w_opt(5:(ndiff+nu)+nx*d:end);
u1_optss = [w_opt(6:(ndiff+nu)+nx*d:end); NaN];
u2_optss = [w_opt(7:(ndiff+nu)+nx*d:end); NaN];
u3_optss = [w_opt(8:(ndiff+nu)+nx*d:end); NaN];

w_optss = [x1_optss, x2_optss, x3_optss, x4_optss, x5_optss, u1_optss, u2_optss, u3_optss];
x_init = w_optss(end,1:ndiff)';
